# LinReg1.py
# Regression of  y = f(x) = ax + b using gradient descent

from gpanel import *
import time

# Set of samples
samples = [[1, 2], [3, 3], [4, 5], [5, 6], [6, 5], [8, 7]]
# Iteration step in both directions
ds = 0.001

def f(x):
    return a * x + b

def showLine():
    global y0, y10
    line(0, y0, 10, y10)  # erase old line
    # New values        
    y0 = f(0)  
    y10 = f(10)    
    line(0, y0, 10, y10) # draw new line

def gradient_a():
    # Derivative with respect to a (slope)
    za = 0 
    for sample in samples:
        xi = sample[0]
        yi = sample[1]
        za += (f(xi) - yi) * xi  
    return 2 * za

def gradient_b():
    # Derivative with respect to b (offset)
    zb = 0
    for sample in samples:
        xi = sample[0]
        yi = sample[1]
        zb += (f(xi) - yi)
    return 2 * zb

def onMousePressed(x, y):
    global iterationNb
    global a, b
    for i in range(100):  # 100 iterations per click
        iterationNb += 1
        anew = a - ds * gradient_a() # descent in a direction   
        bnew = b - ds * gradient_b() # descent in b direction
        # updata a, b
        a = anew
        b = bnew
    showLine()
    showResult()

def showResult():
    title("Iteration # " + str(iterationNb) + ": Slope = "  + \
      str(round(a, 3)) + ", Offset = " + str(round(b, 3))) 
 
makeGPanel(-1, 11, -1, 11, mousePressed = onMousePressed)
drawGrid(0, 10, 0, 10, "gray")
iterationNb = 0
title("Iteration # " + str(iterationNb))
for sample in samples:
    pos(sample)
    fillCircle(0.07)

# Linear fit using library function
xdata = [samples[i][0] for i in range(len(samples))]
ydata = [samples[i][1] for i in range(len(samples))]
a, b = linfit(xdata, ydata)  # linear fit using library function
text(0, 0.7, "Slope = " + str(round(a, 3)))
text(0, 0.3, "Offset = " + str(round(b, 3)))
setColor("green")
lineWidth(3)
line(0, f(0), 10, f(10))

# Prepare iteration
a = 0.5 # first guess for slope
b = 3   # first guess for offset
lineWidth(1)
setColor("red")
setXORMode("white")
y0 = f(0)  
y10 = f(10)    
line(0, y0, 10, y10)
showResult()